@@ -3,7 +3,7 @@
  * Low-level USB interface glue towards libusb.
  *
  * Copyright (C) 2005-2007 Richard A. Low <richard@wentnet.com>
- * Copyright (C) 2005-2008 Linus Walleij <triad@df.lth.se>
+ * Copyright (C) 2005-2011 Linus Walleij <triad@df.lth.se>
  * Copyright (C) 2006-2007 Marcus Meissner
  * Copyright (C) 2007 Ted Bullock
  * Copyright (C) 2008 Chris Bagwell <chris@cnpbagwell.com>
@@ -229,13 +229,18 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
   unsigned char buf[1024], cmd;
   int i;
   int ret;
+  /* This is to indicate if we find some vendor interface */
+  int found_vendor_spec_interface = 0;
 
   /*
-   * Don't examine hubs or HID devices, no point in that.
-   * (Some Kensington mice really don't like this.)
+   * Don't examine devices that are not likely to
+   * contain any MTP interface, update this the day
+   * you find some weird combination...
    */
-  if (dev->descriptor.bDeviceClass == USB_CLASS_HUB ||
-      dev->descriptor.bDeviceClass == USB_CLASS_HID) {
+  if (!(dev->descriptor.bDeviceClass == USB_CLASS_PER_INTERFACE ||
+	dev->descriptor.bDeviceClass == USB_CLASS_COMM ||
+	dev->descriptor.bDeviceClass == USB_CLASS_PTP ||
+	dev->descriptor.bDeviceClass == USB_CLASS_VENDOR_SPEC)) {
     return 0;
   }
 
@@ -269,21 +274,41 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
 	  struct usb_interface_descriptor *intf =
 	    &dev->config[i].interface[j].altsetting[k];
 
+	  /*
+	   * We only want to probe for the OS descriptor if the
+	   * device is USB_CLASS_VENDOR_SPEC or one of the interfaces
+	   * in it is, so flag if we find an interface like this.
+	   */
+	  if (intf->bInterfaceClass == USB_CLASS_VENDOR_SPEC) {
+	    found_vendor_spec_interface = 1;
+	  }
+
 	  /*
 	   * Check for Still Image Capture class with PIMA 15740 protocol,
 	   * also known as PTP
 	   */
+#if 0
 	  if (intf->bInterfaceClass == USB_CLASS_PTP
 	      && intf->bInterfaceSubClass == 0x01
 	      && intf->bInterfaceProtocol == 0x01) {
 	    if (dumpfile != NULL) {
-              fprintf(dumpfile, "Configuration %d, interface %d, altsetting %d:\n", i, j, k);
-	      fprintf(dumpfile, "   Interface implements PTP class,"
-		      " no further probing.\n");
+	      fprintf(dumpfile, "   Found PTP device, check vendor "
+		      "extension...\n");
+	    }
+	    // This is where we may insert code to open a PTP
+	    // session and query the vendor extension ID to see
+	    // if it is 0xffffffff, i.e. MTP according to the spec.
+	    if (was_mtp_extension) {
+	      usb_close(devh);
+	      return 1;
 	    }
-            usb_close(devh);
-            return 1;
 	  }
+#endif
+
+	  /*
+	   * Next we search for the MTP substring in the interface name.
+	   * For example : "RIM MS/MTP" should work.
+	   */
           buf[0] = '\0';
           ret = usb_get_string_simple(devh,
 				      dev->config[i].interface[j].altsetting[k].iInterface,
@@ -291,8 +316,6 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
 				      1024);
 	  if (ret < 3)
 	    continue;
-	  // We search for the MTP string.
-	  // For example : "RIM MS/MTP" should work.
           if (strstr((char *) buf, "MTP") != NULL) {
 	    if (dumpfile != NULL) {
               fprintf(dumpfile, "Configuration %d, interface %d, altsetting %d:\n", i, j, k);
@@ -329,116 +352,139 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
       LIBMTP_INFO("dev->config is NULL in probe_device_descriptor yet dev->descriptor.bNumConfigurations > 0\n");
   }
 
-  /* Read the special descriptor */
-  ret = usb_get_descriptor(devh, 0x03, 0xee, buf, sizeof(buf));
-
-  // Dump it, if requested
-  if (dumpfile != NULL && ret > 0) {
-    fprintf(dumpfile, "Microsoft device descriptor 0xee:\n");
-    data_dump_ascii(dumpfile, buf, ret, 16);
-  }
-
-  /* Check if descriptor length is at least 10 bytes */
-  if (ret < 10) {
-    usb_close(devh);
-    return 0;
-  }
-
-  /* Check if this device has a Microsoft Descriptor */
-  if (!((buf[2] == 'M') && (buf[4] == 'S') &&
-	(buf[6] == 'F') && (buf[8] == 'T'))) {
-    usb_close(devh);
-    return 0;
-  }
-
-  /* Check if device responds to control message 1 or if there is an error */
-  cmd = buf[16];
-  ret = usb_control_msg (devh,
-			 USB_ENDPOINT_IN | USB_RECIP_DEVICE | USB_TYPE_VENDOR,
-			 cmd,
-			 0,
-			 4,
-			 (char *) buf,
-			 sizeof(buf),
-                         USB_TIMEOUT_DEFAULT);
-
-  // Dump it, if requested
-  if (dumpfile != NULL && ret > 0) {
-    fprintf(dumpfile, "Microsoft device response to control message 1, CMD 0x%02x:\n", cmd);
-    data_dump_ascii(dumpfile, buf, ret, 16);
-  }
-
-  /* If this is true, the device either isn't MTP or there was an error */
-  if (ret <= 0x15) {
-    /* TODO: If there was an error, flag it and let the user know somehow */
-    /* if(ret == -1) {} */
-    usb_close(devh);
-    return 0;
-  }
-
-  /* Check if device is MTP or if it is something like a USB Mass Storage
-     device with Janus DRM support */
-  if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
-    usb_close(devh);
-    return 0;
-  }
-
-  /* After this point we are probably dealing with an MTP device */
-
   /*
-   * Check if device responds to control message 2, which is
-   * the extended device parameters. Most devices will just
-   * respond with a copy of the same message as for the first
-   * message, some respond with zero-length (which is OK)
-   * and some with pure garbage. We're not parsing the result
-   * so this is not very important.
+   * Only probe for OS descriptor if the device is vendor specific
+   * or one of the interfaces found is.
    */
-  ret = usb_control_msg (devh,
-			 USB_ENDPOINT_IN | USB_RECIP_DEVICE | USB_TYPE_VENDOR,
-			 cmd,
-			 0,
-			 5,
-			 (char *) buf,
-			 sizeof(buf),
-                         USB_TIMEOUT_DEFAULT);
+  if (dev->descriptor.bDeviceClass == USB_CLASS_VENDOR_SPEC ||
+      found_vendor_spec_interface) {
 
-  // Dump it, if requested
-  if (dumpfile != NULL && ret > 0) {
-    fprintf(dumpfile, "Microsoft device response to control message 2, CMD 0x%02x:\n", cmd);
-    data_dump_ascii(dumpfile, buf, ret, 16);
-  }
+    /* Read the special descriptor */
+    ret = usb_get_descriptor(devh, 0x03, 0xee, buf, sizeof(buf));
 
-  /* If this is true, the device errored against control message 2 */
-  if (ret == -1) {
-    /* TODO: Implement callback function to let managing program know there
-       was a problem, along with description of the problem */
-    LIBMTP_ERROR("Potential MTP Device with VendorID:%04x and "
-	    "ProductID:%04x encountered an error responding to "
-	    "control message 2.\n"
-	    "Problems may arrise but continuing\n",
-	    dev->descriptor.idVendor, dev->descriptor.idProduct);
-  } else if (dumpfile != NULL && ret == 0) {
-    fprintf(dumpfile, "Zero-length response to control message 2 (OK)\n");
-  } else if (dumpfile != NULL) {
-    fprintf(dumpfile, "Device responds to control message 2 with some data.\n");
+    /*
+     * If something failed we're probably stalled to we need
+     * to clear the stall off the endpoint and say this is not
+     * MTP.
+     */
+    if (ret < 0) {
+      /* EP0 is the default control endpoint */
+      usb_clear_halt(devh, 0);
+      usb_close(devh);
+      return 0;
+    }
+
+    // Dump it, if requested
+    if (dumpfile != NULL && ret > 0) {
+      fprintf(dumpfile, "Microsoft device descriptor 0xee:\n");
+      data_dump_ascii(dumpfile, buf, ret, 16);
+    }
+
+    /* Check if descriptor length is at least 10 bytes */
+    if (ret < 10) {
+      usb_close(devh);
+      return 0;
+    }
+
+    /* Check if this device has a Microsoft Descriptor */
+    if (!((buf[2] == 'M') && (buf[4] == 'S') &&
+	  (buf[6] == 'F') && (buf[8] == 'T'))) {
+      usb_close(devh);
+      return 0;
+    }
+
+    /* Check if device responds to control message 1 or if there is an error */
+    cmd = buf[16];
+    ret = usb_control_msg (devh,
+			   USB_ENDPOINT_IN | USB_RECIP_DEVICE | USB_TYPE_VENDOR,
+			   cmd,
+			   0,
+			   4,
+			   (char *) buf,
+			   sizeof(buf),
+			   USB_TIMEOUT_DEFAULT);
+
+    // Dump it, if requested
+    if (dumpfile != NULL && ret > 0) {
+      fprintf(dumpfile, "Microsoft device response to control message 1, CMD 0x%02x:\n", cmd);
+      data_dump_ascii(dumpfile, buf, ret, 16);
+    }
+
+    /* If this is true, the device either isn't MTP or there was an error */
+    if (ret <= 0x15) {
+      /* TODO: If there was an error, flag it and let the user know somehow */
+      /* if(ret == -1) {} */
+      usb_close(devh);
+      return 0;
+    }
+
+    /* Check if device is MTP or if it is something like a USB Mass Storage
+       device with Janus DRM support */
+    if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
+      usb_close(devh);
+      return 0;
+    }
+
+    /* After this point we are probably dealing with an MTP device */
+
+    /*
+     * Check if device responds to control message 2, which is
+     * the extended device parameters. Most devices will just
+     * respond with a copy of the same message as for the first
+     * message, some respond with zero-length (which is OK)
+     * and some with pure garbage. We're not parsing the result
+     * so this is not very important.
+     */
+    ret = usb_control_msg (devh,
+			   USB_ENDPOINT_IN | USB_RECIP_DEVICE | USB_TYPE_VENDOR,
+			   cmd,
+			   0,
+			   5,
+			   (char *) buf,
+			   sizeof(buf),
+			   USB_TIMEOUT_DEFAULT);
+
+    // Dump it, if requested
+    if (dumpfile != NULL && ret > 0) {
+      fprintf(dumpfile, "Microsoft device response to control message 2, CMD 0x%02x:\n", cmd);
+      data_dump_ascii(dumpfile, buf, ret, 16);
+    }
+
+    /* If this is true, the device errored against control message 2 */
+    if (ret == -1) {
+      /* TODO: Implement callback function to let managing program know there
+	 was a problem, along with description of the problem */
+      LIBMTP_ERROR("Potential MTP Device with VendorID:%04x and "
+		   "ProductID:%04x encountered an error responding to "
+		   "control message 2.\n"
+		   "Problems may arrise but continuing\n",
+		   dev->descriptor.idVendor, dev->descriptor.idProduct);
+    } else if (dumpfile != NULL && ret == 0) {
+      fprintf(dumpfile, "Zero-length response to control message 2 (OK)\n");
+    } else if (dumpfile != NULL) {
+      fprintf(dumpfile, "Device responds to control message 2 with some data.\n");
+    }
+    /* Close the USB device handle */
+    usb_close(devh);
+    return 1;
   }
 
   /* Close the USB device handle */
   usb_close(devh);
-  return 1;
+  return 0;
 }
 
 /**
  * This function scans through the connected usb devices on a machine and
  * if they match known Vendor and Product identifiers appends them to the
- * dynamic array mtp_device_list. Be sure to call 
- * <code>free_mtpdevice_list(mtp_device_list)</code> when you are done 
+ * dynamic array mtp_device_list. Be sure to call
+ * <code>free_mtpdevice_list(mtp_device_list)</code> when you are done
  * with it, assuming it is not NULL.
- * @param mtp_device_list dynamic array of pointers to usb devices with MTP 
+ * @param mtp_device_list dynamic array of pointers to usb devices with MTP
  *        properties (if this list is not empty, new entries will be appended
  *        to the list).
  * @return LIBMTP_ERROR_NONE implies that devices have been found, scan the list
- *        appropriately. LIBMTP_ERROR_NO_DEVICE_ATTACHED implies that no 
+ *        appropriately. LIBMTP_ERROR_NO_DEVICE_ATTACHED implies that no
  *        devices have been found.
  */
 static LIBMTP_error_number_t get_mtp_usb_device_list(mtpdevice_list_t ** mtp_device_list)
@@ -458,8 +504,8 @@ static LIBMTP_error_number_t get_mtp_usb_device_list(mtpdevice_list_t ** mtp_dev
           if(dev->descriptor.idVendor == mtp_device_table[i].vendor_id &&
             dev->descriptor.idProduct == mtp_device_table[i].product_id) {
             /* Append this usb device to the MTP device list */
-            *mtp_device_list = append_to_mtpdevice_list(*mtp_device_list, 
-							dev, 
+            *mtp_device_list = append_to_mtpdevice_list(*mtp_device_list,
+							dev,
 							bus->location);
             found = 1;
             break;
@@ -469,7 +515,7 @@ static LIBMTP_error_number_t get_mtp_usb_device_list(mtpdevice_list_t ** mtp_dev
         if (!found) {
           if (probe_device_descriptor(dev, NULL)) {
             /* Append this usb device to the MTP USB Device List */
-            *mtp_device_list = append_to_mtpdevice_list(*mtp_device_list, 
+            *mtp_device_list = append_to_mtpdevice_list(*mtp_device_list,
 							dev,
 							bus->location);
           }
@@ -489,7 +535,7 @@ static LIBMTP_error_number_t get_mtp_usb_device_list(mtpdevice_list_t ** mtp_dev
       }
     }
   }
-  
+
   /* If nothing was found we end up here. */
   if(*mtp_device_list == NULL) {
     return LIBMTP_ERROR_NO_DEVICE_ATTACHED;
@@ -497,22 +543,50 @@ static LIBMTP_error_number_t get_mtp_usb_device_list(mtpdevice_list_t ** mtp_dev
   return LIBMTP_ERROR_NONE;
 }
 
+/**
+ * Checks if a specific device with a certain bus and device
+ * number has an MTP type device descriptor.
+ *
+ * @param busno the bus number of the device to check
+ * @param deviceno the device number of the device to check
+ * @return 1 if the device is MTP else 0
+ */
+int LIBMTP_Check_Specific_Device(int busno, int devno)
+{
+  struct usb_bus *bus = init_usb();
+  for (; bus != NULL; bus = bus->next) {
+    struct usb_device *dev = bus->devices;
+    if (bus->location != busno)
+      continue;
+
+    for (; dev != NULL; dev = dev->next) {
+
+      if (dev->devnum != devno)
+	continue;
+
+      if (probe_device_descriptor(dev, NULL))
+	return 1;
+    }
+  }
+  return 0;
+}
+
 /**
  * Detect the raw MTP device descriptors and return a list of
  * of the devices found.
- * 
+ *
  * @param devices a pointer to a variable that will hold
  *        the list of raw devices found. This may be NULL
  *        on return if the number of detected devices is zero.
  *        The user shall simply <code>free()</code> this
  *        variable when finished with the raw devices,
  *        in order to release memory.
- * @param numdevs a pointer to an integer that will hold 
+ * @param numdevs a pointer to an integer that will hold
  *        the number of devices in the list. This may
  *        be 0.
  * @return 0 if successful, any other value means failure.
  */
-LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices, 
+LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
 			      int * numdevs)
 {
   mtpdevice_list_t *devlist = NULL;
@@ -573,7 +647,7 @@ LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
 	retdevs[i].device_entry.device_flags = mtp_device_table[j].device_flags;
 
 	// This device is known to the developers
-	LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is a %s %s.\n", 
+	LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is a %s %s.\n",
 		i,
 		dev->libusb_device->descriptor.idVendor,
 		dev->libusb_device->descriptor.idProduct,
@@ -584,7 +658,7 @@ LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
     }
     if (!device_known) {
       // This device is unknown to the developers
-      LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is UNKNOWN.\n", 
+      LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is UNKNOWN.\n",
 	      i,
 	      dev->libusb_device->descriptor.idVendor,
 	      dev->libusb_device->descriptor.idProduct);
@@ -601,7 +675,7 @@ LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
     retdevs[i].devnum = dev->libusb_device->devnum;
     i++;
     dev = dev->next;
-  }  
+  }
   *devices = retdevs;
   *numdevs = i;
   free_mtpdevice_list(devlist);
@@ -620,7 +694,7 @@ void dump_usbinfo(PTP_USB *ptp_usb)
 #ifdef LIBUSB_HAS_GET_DRIVER_NP
   char devname[0x10];
   int res;
-  
+
   devname[0] = '\0';
   res = usb_get_driver_np(ptp_usb->handle, (int) ptp_usb->interface, devname, sizeof(devname));
   if (devname[0] != '\0') {
@@ -688,7 +762,7 @@ libusb_glue_debug (PTPParams *params, const char *format, ...)
 
 static void
 libusb_glue_error (PTPParams *params, const char *format, ...)
-{  
+{
         va_list args;
 
         va_start (args, format);
@@ -711,7 +785,7 @@ libusb_glue_error (PTPParams *params, const char *format, ...)
  * reveals that WMP / Windows is probably using an algorithm like this
  * for large transfers:
  *
- * 1. Send the command (0x0c bytes) if headers are split, else, send 
+ * 1. Send the command (0x0c bytes) if headers are split, else, send
  *    command plus sizeof(endpoint) - 0x0c bytes.
  * 2. Send first packet, max size to be sizeof(endpoint) but only when using
  *    split headers. Else goto 3.
@@ -745,7 +819,6 @@ ptp_read_func (
   // This is the largest block we'll need to read in.
   bytes = malloc(CONTEXT_BLOCK_SIZE);
   while (curread < size) {
-    
 
     LIBMTP_USB_DEBUG("Remaining size to read: 0x%04lx bytes\n", size - curread);
 
@@ -776,7 +849,7 @@ ptp_read_func (
     result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, (char*)bytes, toread, ptp_usb->timeout);
 
     LIBMTP_USB_DEBUG("Result of read: 0x%04x\n", result);
-        
+
     if (result < 0) {
       return PTP_ERROR_IO;
     }
@@ -786,7 +859,7 @@ ptp_read_func (
       LIBMTP_USB_DEBUG("Zero Read\n");
     else
       LIBMTP_USB_DATA(bytes, result, 16);
-    
+
     // want to discard extra byte
     if (expect_terminator_byte && result == toread)
     {
@@ -794,11 +867,11 @@ ptp_read_func (
 
       result--;
     }
-    
+
     int putfunc_ret = handler->putfunc(NULL, handler->priv, result, bytes, &written);
     if (putfunc_ret != PTP_RC_OK)
       return putfunc_ret;
-    
+
     ptp_usb->current_transfer_complete += result;
     curread += result;
 
@@ -818,17 +891,17 @@ ptp_read_func (
 	  return PTP_ERROR_CANCEL;
 	}
       }
-    }  
+    }
 
     if (result < toread) /* short reads are common */
       break;
   }
   if (readbytes) *readbytes = curread;
   free (bytes);
-  
+
   // there might be a zero packet waiting for us...
-  if (readzero && 
-      !FLAG_NO_ZERO_READS(ptp_usb) && 
+  if (readzero &&
+      !FLAG_NO_ZERO_READS(ptp_usb) &&
       curread % ptp_usb->outep_maxpacket == 0) {
     char temp;
     int zeroresult = 0;
@@ -840,7 +913,7 @@ ptp_read_func (
     if (zeroresult != 0)
       LIBMTP_INFO("LIBMTP panic: unable to read in zero packet, response 0x%04x", zeroresult);
   }
-  
+
   return PTP_RC_OK;
 }
 
@@ -857,7 +930,7 @@ ptp_write_func (
   unsigned long curwrite = 0;
   unsigned char *bytes;
 
-  // This is the largest block we'll need to read in.  
+  // This is the largest block we'll need to read in.
   bytes = malloc(CONTEXT_BLOCK_SIZE);
   if (!bytes) {
     return PTP_ERROR_IO;
@@ -915,7 +988,6 @@ ptp_write_func (
   if (written) {
     *written = curwrite;
   }
-  
 
   // If this is the last transfer send a zero write if required
   if (ptp_usb->current_transfer_complete >= ptp_usb->current_transfer_total) {
@@ -927,7 +999,7 @@ ptp_write_func (
       result=USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char *)"x",0,ptp_usb->timeout);
     }
   }
-    
+
   if (result < 0)
     return PTP_ERROR_IO;
   return PTP_RC_OK;
@@ -1068,7 +1140,7 @@ ptp_usb_sendreq (PTPParams* params, PTPContainer* req)
 		ret = PTP_ERROR_IO;
 	}
 	if (written != towrite && ret != PTP_ERROR_CANCEL && ret != PTP_ERROR_IO) {
-		libusb_glue_error (params, 
+		libusb_glue_error (params,
 			"PTP: request code 0x%04x sending req wrote only %ld bytes instead of %d",
 			req->Code, written, towrite
 		);
@@ -1206,7 +1278,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 				ret = dtoh16(usbdata.code);
 				// This filters entirely insane garbage return codes, but still
 				// makes it possible to return error codes in the code field when
-				// getting data. It appears Windows ignores the contents of this 
+				// getting data. It appears Windows ignores the contents of this
 				// field entirely.
 				if (ret < PTP_RC_Undefined || ret > PTP_RC_SpecificationOfDestinationUnsupported) {
 					libusb_glue_debug (params, "ptp2/ptp_usb_getdata: detected a broken "
@@ -1270,7 +1342,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 			} else if(!FLAG_NO_ZERO_READS(ptp_usb) &&
 				  (rlen - dtoh32(usbdata.length) == 1)) {
 			  libusb_glue_debug (params, "ptp2/ptp_usb_getdata: read %d bytes "
-				     "too much, expect problems!", 
+				     "too much, expect problems!",
 				     rlen - dtoh32(usbdata.length));
 			}
 			rlen = packlen;
@@ -1541,7 +1613,7 @@ static int init_ptp_usb (PTPParams* params, PTP_USB* ptp_usb, struct usb_device*
       return -1;
     }
 #endif
-    // It seems like on kernel 2.6.31 if we already have it open on another 
+    // It seems like on kernel 2.6.31 if we already have it open on another
     // pthread in our app, we'll get an error if we try to claim it again,
     // but that error is harmless because our process already claimed the interface
     usbresult = usb_claim_interface(device_handle, (int) ptp_usb->interface);
@@ -1681,9 +1753,6 @@ static void close_usb(PTP_USB* ptp_usb)
     usb_resetep(ptp_usb->handle, ptp_usb->outep);
     usb_release_interface(ptp_usb->handle, (int) ptp_usb->interface);
   }
-  // Brutally reset device
-  // TODO: is this good on the Mac too?
-  usb_reset(ptp_usb->handle);
   usb_close(ptp_usb->handle);
 }
 
@@ -1725,27 +1794,24 @@ static int find_interface_and_endpoints(struct usb_device *dev,
       // Loop over the three endpoints to locate two bulk and
       // one interrupt endpoint and FAIL if we cannot, and continue.
       for (k = 0; k < no_ep; k++) {
-	if (ep[k].bmAttributes==USB_ENDPOINT_TYPE_BULK)	{
-	  if ((ep[k].bEndpointAddress&USB_ENDPOINT_DIR_MASK)==
-	      USB_ENDPOINT_DIR_MASK)
-	    {
-	      *inep=ep[k].bEndpointAddress;
-	      *inep_maxpacket=ep[k].wMaxPacketSize;
-	      found_inep = 1;
-	    }
-	  if ((ep[k].bEndpointAddress&USB_ENDPOINT_DIR_MASK)==0)
-	    {
-	      *outep=ep[k].bEndpointAddress;
-	      *outep_maxpacket=ep[k].wMaxPacketSize;
-	      found_outep = 1;
-	    }
-	} else if (ep[k].bmAttributes==USB_ENDPOINT_TYPE_INTERRUPT){
-	  if ((ep[k].bEndpointAddress&USB_ENDPOINT_DIR_MASK)==
-	      USB_ENDPOINT_DIR_MASK)
-	    {
-	      *intep=ep[k].bEndpointAddress;
-	      found_intep = 1;
-	    }
+	if (ep[k].bmAttributes == USB_ENDPOINT_TYPE_BULK) {
+	  if ((ep[k].bEndpointAddress & USB_ENDPOINT_DIR_MASK) ==
+	      USB_ENDPOINT_DIR_MASK) {
+	    *inep = ep[k].bEndpointAddress;
+	    *inep_maxpacket = ep[k].wMaxPacketSize;
+	    found_inep = 1;
+	  }
+	  if ((ep[k].bEndpointAddress & USB_ENDPOINT_DIR_MASK) == 0) {
+	    *outep = ep[k].bEndpointAddress;
+	    *outep_maxpacket = ep[k].wMaxPacketSize;
+	    found_outep = 1;
+	  }
+	} else if (ep[k].bmAttributes == USB_ENDPOINT_TYPE_INTERRUPT) {
+	  if ((ep[k].bEndpointAddress & USB_ENDPOINT_DIR_MASK) ==
+	      USB_ENDPOINT_DIR_MASK) {
+	    *intep = ep[k].bEndpointAddress;
+	    found_intep = 1;
+	  }
 	}
       }
       if (found_inep && found_outep && found_intep)
@@ -1764,7 +1830,7 @@ static int find_interface_and_endpoints(struct usb_device *dev,
  * @param usbinfo a pointer to the new usbinfo.
  * @return an error code.
  */
-LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device, 
+LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device,
 					   PTPParams *params,
 					   void **usbinfo)
 {
@@ -1844,16 +1910,22 @@ LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device,
    * have not used LIBMTP_Release_Device on exit
    */
   if ((ret = ptp_opensession(params, 1)) == PTP_ERROR_IO) {
-    LIBMTP_ERROR("PTP_ERROR_IO: Trying again after re-initializing USB interface\n");
+    LIBMTP_ERROR("PTP_ERROR_IO: failed to open session, trying again after resetting USB interface\n");
     close_usb(ptp_usb);
 
+    LIBMTP_ERROR("LIBMTP libusb: Attempt to reset device\n");
+    usb_reset(ptp_usb->handle);
+
     if(init_ptp_usb(params, ptp_usb, libusb_device) <0) {
-      LIBMTP_ERROR("LIBMTP PANIC: Could not open session on device\n");
+      LIBMTP_ERROR("LIBMTP PANIC: Could not init USB on second attempt\n");
       return LIBMTP_ERROR_CONNECTING;
     }
 
     /* Device has been reset, try again */
-    ret = ptp_opensession(params, 1);
+    if ((ret = ptp_opensession(params, 1)) == PTP_ERROR_IO) {
+      LIBMTP_ERROR("LIBMTP PANIC: failed to open session on second attempt\n");
+      return LIBMTP_ERROR_CONNECTING;
+    }
   }
 
   /* Was the transaction id invalid? Try again */
